require "acts_as_token/version"

module ActsAsToken
  extend ActiveSupport::Concern

  included do
    before_save :ensure_token_id!
  end

  module ClassMethods

    def acts_as_token(column_name: :token_id, length: 10)
      unless column_names.include? column_name.to_s
        raise "#{ self.name } does not have a column #{ column_name }"
      else
        instance_variable_set('@_column_name', column_name.to_sym)
        instance_variable_set('@_length', length.to_i)
      end
    end

  end

  def ensure_token_id!
    self.send("#{ column_name }=", generate_token_id)
  end

  def generate_token_id
    loop do
      token = rand(10 ** length_of_token).to_s.rjust(length_of_token,'0')
      break token unless self.class.find_by("#{ column_name }" => token)
    end
  end

  def length_of_token
    self.class.instance_variable_get('@_length')
  end

  def column_name
    self.class.instance_variable_get('@_column_name')
  end

end

class ActiveRecord::Base
  include ActsAsToken
end
